<?php

namespace Drupal\paragraphs_selection\Plugin\EntityReferenceSelection;

use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\Plugin\EntityReferenceSelection\DefaultSelection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\NestedArray;
use Drupal\paragraphs\Plugin\EntityReferenceSelection\ParagraphSelection;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Default plugin implementation of the Entity Reference Selection plugin.
 *
 * @EntityReferenceSelection(
 *   id = "paragraph_reverse",
 *   label = @Translation("Reverse Paragraphs Selection"),
 *   group = "paragraph_reverse",
 *   entity_types = {"paragraph"},
 *   weight = 5,
 * )
 */
class ReverseParagraphSelection extends ParagraphSelection {

  /**
   * Entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
//  public $entityTypeBundleInfo;

  /**
   * ParagraphSelection constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   Entity type bundle info service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityManagerInterface $entity_manager, ModuleHandlerInterface $module_handler, AccountInterface $current_user, EntityTypeBundleInfoInterface $entity_type_bundle_info) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_manager, $module_handler, $current_user, $entity_type_bundle_info);

//    $this->entityTypeBundleInfo = $entity_type_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity.manager'),
      $container->get('module_handler'),
      $container->get('current_user'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $field = $form_state->getFormObject()->getEntity()->id();

    $selection = [
      'field' => $field,
    ];

    $form['selection'] = [
      '#type' => 'value',
      '#value' => $selection,
    ];

    return $form;
  }

  /**
   * Returns the sorted allowed types for the field.
   *
   * @return array
   *   A list of arrays keyed by the paragraph type machine name
   *   with the following properties.
   *     - label: The label of the paragraph type.
   *     - weight: The weight of the paragraph type.
   */
  public function getSortedAllowedTypes() {

    if (isset($this->configuration, $this->configuration['selection'], $this->configuration['selection']['field'], $this->configuration['target_type'])) {

      $field = $this->configuration['selection']['field'];
      $target_type = $this->configuration['target_type'];

      if ($field && $target_type) {
        $bundle_entity_type = $this->entityTypeManager->getDefinition($target_type)->getBundleEntityType();

        $return_bundles = [];
        $bundles = $this->entityTypeManager->getStorage($bundle_entity_type)->loadMultiple();

        foreach ($bundles as $bundle) {
          $selection_configuration = $bundle->getThirdPartySetting('paragraphs_selection', 'selection');
          if (isset($selection_configuration['fields']) && count($selection_configuration['fields']) > 0) {
            foreach ($selection_configuration['fields'] as $selection_field) {

              if (isset($selection_field['name']) && $selection_field['name'] === $field) {
                $return_bundles[$bundle->id()] = [
                  'label' => $bundle->label(),
                  'weight' => isset($selection_field['weight']) ? $selection_field['weight'] : 50,
                ];
              }

            }
          }
        }

        // Sort bundle list by weight.
        $return_bundles = $this->sortBundles($return_bundles, 'weight');

        return $return_bundles;
      }
    }

    return parent::getSortedAllowedTypes();

  }

  protected function sortBundles($a, $subkey) {
    foreach($a as $k=>$v) {
      $b[$k] = strtolower($v[$subkey]);
    }
    asort($b);
    foreach($b as $key=>$val) {
      $c[$key] = $a[$key];
    }
    return $c;
  }

}
